% Versuch 3_1
% C. Roppel, HS Schmalkalden
% MATLAB- und Scilab-bungen zum Buch "Grundlagen der Nachrichtentechnik", Hanser-Verlag, 2023
% Getestet mit MATLAB R2020a
%%
clear
clc
close all

%% Parameter
alpha = 0.5;    % Roll-off-Faktor
os = 8;         % Samples pro Symbol (Oversampling Ratio)
dly_rc = 3;     % Verzgerung des Wurzel-Kosinus-roll-off-Filters
nSym = 1000;    % Anzahl Symbole
m = 4;          % m-wertige Symbole
k = log2(m);    % k Bit pro Symbol
lambda = pi/4;  % QPSK Phasenoffset

%% Wurzel-Kosinus-roll-off-Filter
psrc = rcosdesign(alpha, 2*dly_rc, os, 'sqrt');

%% QPSK-Modulator und Pulsformfilter
rng(1234);  % Initialisiere Zufallsgenerator
symbols = randi([0 (m - 1)], nSym, 1);  % Spaltenvektor der Lnge nSym mit Zufallszahlen im Bereich 0 bis m-1

xmod = pskmod(symbols, m, lambda);  % Modulation

xmod = upsample(xmod, os);  % Einfgen von os - 1 Nullen zwischen den Symbolen
x = conv(psrc, xmod);  % Faltung (convolution)
x = x(dly_rc*os + 1:end - dly_rc*os);  % Entfernen der ersten und letzten Werte

%% Mehrwegekanal
dt = 2;   % Laufzeitdifferenz (in Samples)
a = 0.8;  % Dmpfung des indirekten Pfades
b = 1;    % Produkt f_c * t_0

% Impulsantwort
hKanal = zeros(1, dt + 1);
hKanal(1) = 1;
hKanal(dt + 1) = a*exp(-j*2*pi*b);

% bertragungsfunktion
f = (os*(-256:255)/512);
HKanal = 1 + a*exp(-j*2*pi*b)*exp(-j*2*pi*f*dt/os);
figure;
plot(f, abs(HKanal));
xlabel('f in Hz');
ylabel('|H_K(f)|');

% Faltung mit der Kanalimpulsantwort
r = conv(hKanal, x);

figure; 
periodogram(r, [], [], os, 'centered');

%% Signalangepasstes Filter
ye = conv(psrc, r);
ye = ye(dly_rc*os + 1:end - dly_rc*os);

eyediagram(ye, 2*os, 2);
